<?php

namespace Drupal\smart_date\Plugin\FullcalendarViewProcessor;

use Drupal\fullcalendar_view\Plugin\FullcalendarViewProcessorBase;

/**
 * Smart Date plugin.
 *
 * @FullcalendarViewProcessor(
 *   id = "fullcalendar_view_smart_date",
 *   label = @Translation("Smart date processor"),
 *   field_types = {
 *     "smartdate"
 *   }
 * )
 */
class SmartDateProcessor extends FullcalendarViewProcessorBase {

  /**
   * @inheritDoc
   *
   * processing view results of fullcalendar_view for a smart date field
   *
   * ToDo:
   * - handling of drag and drop updates full calendar
   * - timezone handling
   * - smart date recurring events handling (should be roughly working)
   * - fullcalender_view recurring events handling (not considered yet, maybe
   * this is not to be supported by smart date anyway
   */
  public function process(array &$variables) {
    $view = $variables['view'];
    $fields = $view->field;
    $options =  $view->style_plugin->options;
    $start_field = $options['start'];
    $start_field_options = $fields[$start_field]->options;
    $multiple = $fields[$start_field]->multiple;
    if (strpos($start_field_options['type'], 'smartdate') === 0) {
      $entries = isset($variables['#attached']['drupalSettings']['fullCalendarView']) ? $variables['#attached']['drupalSettings']['fullCalendarView'] : [];
      if (!empty($entries)) {
        if ($multiple && $start_field_options['group_rows'] == TRUE) {
          $messenger = \Drupal::messenger();
          $messenger->addMessage('Do not group rows', $messenger::TYPE_WARNING);
        }
        else {
          $entry_counter = 0;
          foreach ($view->result as $key => $row) {
            $current_entity = $row->_entity;
            if ($multiple) {
              $type = $row->_entity->getEntityTypeId();
              $delta_key = $type . '__' . $start_field . '_delta';
              $delta = $row->{$delta_key};
            }
            else {
              $delta = 0;
            }
            $values = $current_entity->get($start_field)->getValue();
            if (!empty($values)) {
              $start = $values[$delta]['value'];
              $end = $values[$delta]['end_value'];
              if (!empty($start)) {
                if (date('Hi', $start) == "0000" && date('Hi', $end) == "2359") {
                  $start = date('Y-m-d', $start);
                  // The end date is inclusive for a all day event in full calendar,
                  // which is not what we want. So we need one day offset.
                  $end = date('Y-m-d', $end + (60 * 60 * 24));
                }
                else {
                  $start = date(DATE_ATOM, $start);
                  $end = date(DATE_ATOM, $end);
                }
                $entries[$entry_counter]['start'] = $start;
                $entries[$entry_counter]['end'] = $end;
                $entry_counter++;
              }
            }
          }
        }
      }
      // Update the entries
      if ($entries) {
        $variables['#attached']['drupalSettings']['fullCalendarView'] = $entries;
      }
    }
  }

}
